@@ -37,10 +37,17 @@
  * The 0.49 value is used for rounding correctness, because ceil()
  * rounds up if the number is > 0.0.
  */
-#define PROGRESS_ROUNDED(x) (ceil (((x) * 100) - 0.49) / 100)
+#define PROGRESS_ROUNDED(x) ((x) < 0.01 ? 0.00 : (ceil (((x) * 100) - 0.49) / 100))
 
 #define TRACKER_SERVICE "org.freedesktop.Tracker1"
 
+#ifdef MINER_STATUS_ENABLE_TRACE
+#warning Miner status traces are enabled
+#define trace(message, ...) g_debug (message, ##__VA_ARGS__)
+#else
+#define trace(...)
+#endif /* MINER_STATUS_ENABLE_TRACE */
+
 /**
  * SECTION:tracker-miner
  * @short_description: Abstract base class for data miners
@@ -80,6 +87,11 @@ static const gchar introspection_xml[] =
   "      <arg type='s' name='reason' direction='in' />"
   "      <arg type='i' name='cookie' direction='out' />"
   "    </method>"
+  "    <method name='PauseForProcess'>"
+  "      <arg type='s' name='application' direction='in' />"
+  "      <arg type='s' name='reason' direction='in' />"
+  "      <arg type='i' name='cookie' direction='out' />"
+  "    </method>"
   "    <method name='Resume'>"
   "      <arg type='i' name='cookie' direction='in' />"
   "    </method>"
@@ -111,12 +123,15 @@ struct _TrackerMinerPrivate {
 	guint registration_id;
 	gchar *full_name;
 	gchar *full_path;
+	guint update_id;
 };
 
 typedef struct {
 	gint cookie;
 	gchar *application;
 	gchar *reason;
+	gchar *watch_name;
+	guint watch_name_id;
 } PauseData;
 
 enum {
@@ -153,7 +168,9 @@ static gboolean   miner_initable_init          (GInitable              *initable
                                                 GError                **error);
 static void       pause_data_destroy           (gpointer                data);
 static PauseData *pause_data_new               (const gchar            *application,
-                                                const gchar            *reason);
+                                                const gchar            *reason,
+                                                const gchar            *watch_name,
+                                                guint                   watch_name_id);
 static void       handle_method_call           (GDBusConnection        *connection,
                                                 const gchar            *sender,
                                                 const gchar            *object_path,
@@ -328,8 +345,8 @@ tracker_miner_class_init (TrackerMinerClass *klass)
 	                                 g_param_spec_string ("status",
 	                                                      "Status",
 	                                                      "Translatable string with status description",
-	                                                      NULL,
-	                                                      G_PARAM_READWRITE));
+	                                                      "Idle",
+	                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 	g_object_class_install_property (object_class,
 	                                 PROP_PROGRESS,
 	                                 g_param_spec_double ("progress",
@@ -474,6 +491,7 @@ static void
 tracker_miner_init (TrackerMiner *miner)
 {
 	TrackerMinerPrivate *priv;
+
 	miner->priv = priv = TRACKER_MINER_GET_PRIVATE (miner);
 
 	priv->pauses = g_hash_table_new_full (g_direct_hash,
@@ -482,9 +500,13 @@ tracker_miner_init (TrackerMiner *miner)
 	                                      pause_data_destroy);
 }
 
-static void
-miner_update_progress (TrackerMiner *miner)
+static gboolean
+miner_update_progress_cb (gpointer data)
 {
+	TrackerMiner *miner = data;
+
+	trace ("(Miner:'%s') UPDATE PROGRESS SIGNAL", miner->priv->name);
+
 	g_signal_emit (miner, signals[PROGRESS], 0,
 	               miner->priv->status,
 	               miner->priv->progress);
@@ -500,6 +522,10 @@ miner_update_progress (TrackerMiner *miner)
 		                                              miner->priv->progress),
 		                               NULL);
 	}
+
+	miner->priv->update_id = 0;
+
+	return FALSE;
 }
 
 static void
@@ -510,6 +536,15 @@ miner_set_property (GObject      *object,
 {
 	TrackerMiner *miner = TRACKER_MINER (object);
 
+	/* Quite often, we see status of 100% and still have
+	 * status messages saying Processing... which is not
+	 * true. So we use an idle timeout to help that situation.
+	 * Additionally we can't force both properties are correct
+	 * with the GObject API, so we have to do some checks our
+	 * selves. The g_object_bind_property() API also isn't
+	 * sufficient here.
+	 */
+
 	switch (prop_id) {
 	case PROP_NAME:
 		g_free (miner->priv->name);
@@ -519,6 +554,11 @@ miner_set_property (GObject      *object,
 		const gchar *new_status;
 
 		new_status = g_value_get_string (value);
+
+		trace ("(Miner:'%s') Set property:'status' to '%s'",
+		       miner->priv->name,
+		       new_status);
+
 		if (miner->priv->status && new_status &&
 		    strcmp (miner->priv->status, new_status) == 0) {
 			/* Same, do nothing */
@@ -527,13 +567,39 @@ miner_set_property (GObject      *object,
 
 		g_free (miner->priv->status);
 		miner->priv->status = g_strdup (new_status);
-		miner_update_progress (miner);
+
+		/* Check progress matches special statuses */
+		if (new_status != NULL) {
+			if (g_ascii_strcasecmp (new_status, "Initializing") == 0 &&
+			    miner->priv->progress != 0.0) {
+				trace ("(Miner:'%s') Set progress to 0.0 from status:'Initializing'",
+				       miner->priv->name);
+				miner->priv->progress = 0.0;
+			} else if (g_ascii_strcasecmp (new_status, "Idle") == 0 &&
+			           miner->priv->progress != 1.0) {
+				trace ("(Miner:'%s') Set progress to 1.0 from status:'Idle'",
+				       miner->priv->name);
+				miner->priv->progress = 1.0;
+			}
+		}
+
+		if (miner->priv->update_id == 0) {
+			miner->priv->update_id = g_idle_add_full (G_PRIORITY_HIGH_IDLE,
+			                                          miner_update_progress_cb,
+			                                          miner,
+			                                          NULL);
+		}
+
 		break;
 	}
 	case PROP_PROGRESS: {
 		gdouble new_progress;
 
 		new_progress = PROGRESS_ROUNDED (g_value_get_double (value));
+		trace ("(Miner:'%s') Set property:'progress' to '%2.2f' (%2.2f before rounded)",
+		         miner->priv->name,
+		         new_progress,
+		         g_value_get_double (value));
 
 		/* NOTE: We don't round the current progress before
 		 * comparison because we use the rounded value when
@@ -547,7 +613,33 @@ miner_set_property (GObject      *object,
 		}
 
 		miner->priv->progress = new_progress;
-		miner_update_progress (miner);
+
+		/* Check status matches special progress values */
+		if (new_progress == 0.0) {
+			if (miner->priv->status == NULL ||
+			    g_ascii_strcasecmp (miner->priv->status, "Initializing") != 0) {
+				trace ("(Miner:'%s') Set status:'Initializing' from progress:0.0",
+				       miner->priv->name);
+				g_free (miner->priv->status);
+				miner->priv->status = g_strdup ("Initializing");
+			}
+		} else if (new_progress == 1.0) {
+			if (miner->priv->status == NULL ||
+			    g_ascii_strcasecmp (miner->priv->status, "Idle") != 0) {
+				trace ("(Miner:'%s') Set status:'Idle' from progress:1.0",
+				       miner->priv->name);
+				g_free (miner->priv->status);
+				miner->priv->status = g_strdup ("Idle");
+			}
+		}
+
+		if (miner->priv->update_id == 0) {
+			miner->priv->update_id = g_idle_add_full (G_PRIORITY_HIGH_IDLE,
+			                                          miner_update_progress_cb,
+			                                          miner,
+			                                          NULL);
+		}
+
 		break;
 	}
 	default:
@@ -582,7 +674,9 @@ miner_get_property (GObject    *object,
 
 static PauseData *
 pause_data_new (const gchar *application,
-                const gchar *reason)
+                const gchar *reason,
+                const gchar *watch_name,
+                guint        watch_name_id)
 {
 	PauseData *data;
 	static gint cookie = 1;
@@ -592,6 +686,8 @@ pause_data_new (const gchar *application,
 	data->cookie = cookie++;
 	data->application = g_strdup (application);
 	data->reason = g_strdup (reason);
+	data->watch_name = g_strdup (watch_name);
+	data->watch_name_id = watch_name_id;
 
 	return data;
 }
@@ -603,6 +699,12 @@ pause_data_destroy (gpointer data)
 
 	pd = data;
 
+	if (pd->watch_name_id) {
+		g_bus_unwatch_name (pd->watch_name_id);
+	}
+
+	g_free (pd->watch_name);
+
 	g_free (pd->reason);
 	g_free (pd->application);
 
@@ -755,15 +857,58 @@ tracker_miner_get_n_pause_reasons (TrackerMiner *miner)
 	return g_hash_table_size (miner->priv->pauses);
 }
 
+static void
+pause_process_disappeared_cb (GDBusConnection *connection,
+                              const gchar     *name,
+                              gpointer         user_data)
+{
+	TrackerMiner *miner;
+	PauseData *pd = NULL;
+	GError *error = NULL;
+	GHashTableIter iter;
+	gpointer key, value;
+
+	miner = user_data;
+
+	g_message ("Process with name:'%s' has disappeared", name);
+
+	g_hash_table_iter_init (&iter, miner->priv->pauses);
+	while (g_hash_table_iter_next (&iter, &key, &value)) {
+		PauseData *pd_iter = value;
+
+		if (g_strcmp0 (name, pd_iter->watch_name) == 0) {
+			pd = pd_iter;
+			break;
+		}
+	}
+
+	if (!pd) {
+		g_critical ("Could not find PauseData for process with name:'%s'", name);
+		return;
+	}
+
+	/* Resume */
+	g_message ("Resuming pause associated with process");
+
+	tracker_miner_resume (miner, pd->cookie, &error);
+
+	if (error) {
+		g_warning ("Could not resume miner, %s", error->message);
+		g_error_free (error);
+	}
+}
+
 static gint
-tracker_miner_pause_internal (TrackerMiner  *miner,
-                              const gchar   *application,
-                              const gchar   *reason,
-                              GError       **error)
+miner_pause_internal (TrackerMiner  *miner,
+                      const gchar   *application,
+                      const gchar   *reason,
+                      const gchar   *calling_name,
+                      GError       **error)
 {
 	PauseData *pd;
 	GHashTableIter iter;
 	gpointer key, value;
+	guint watch_name_id = 0;
 
 	/* Check this is not a duplicate pause */
 	g_hash_table_iter_init (&iter, miner->priv->pauses);
@@ -779,7 +924,18 @@ tracker_miner_pause_internal (TrackerMiner  *miner,
 		}
 	}
 
-	pd = pause_data_new (application, reason);
+	if (calling_name) {
+		g_message ("Watching process with name:'%s'", calling_name);
+		watch_name_id = g_bus_watch_name (G_BUS_TYPE_SESSION,
+		                                  calling_name,
+		                                  G_BUS_NAME_WATCHER_FLAGS_NONE,
+		                                  NULL,
+		                                  pause_process_disappeared_cb,
+		                                  miner,
+		                                  NULL);
+	}
+
+	pd = pause_data_new (application, reason, calling_name, watch_name_id);
 
 	g_hash_table_insert (miner->priv->pauses,
 	                     GINT_TO_POINTER (pd->cookie),
@@ -834,7 +990,7 @@ tracker_miner_pause (TrackerMiner  *miner,
 		application = miner->priv->name;
 	}
 
-	return tracker_miner_pause_internal (miner, application, reason, error);
+	return miner_pause_internal (miner, application, reason, NULL, error);
 }
 
 /**
@@ -925,7 +1081,7 @@ tracker_miner_get_dbus_connection (TrackerMiner *miner)
  *
  * Since: 0.10
  **/
-G_CONST_RETURN gchar *
+const gchar *
 tracker_miner_get_dbus_full_name (TrackerMiner *miner)
 {
 	return miner->priv->full_name;
@@ -941,7 +1097,7 @@ tracker_miner_get_dbus_full_name (TrackerMiner *miner)
  *
  * Since: 0.10
  **/
-G_CONST_RETURN gchar *
+const gchar *
 tracker_miner_get_dbus_full_path (TrackerMiner *miner)
 {
 	return miner->priv->full_path;
@@ -952,6 +1108,10 @@ miner_finalize (GObject *object)
 {
 	TrackerMiner *miner = TRACKER_MINER (object);
 
+	if (miner->priv->update_id != 0) {
+		g_source_remove (miner->priv->update_id);
+	}
+
 	if (miner->priv->watch_name_id != 0) {
 		g_bus_unwatch_name (miner->priv->watch_name_id);
 	}
@@ -1055,7 +1215,48 @@ handle_method_call_pause (TrackerMiner          *miner,
 	                                        application,
 	                                        reason);
 
-	cookie = tracker_miner_pause_internal (miner, application, reason, &local_error);
+	cookie = miner_pause_internal (miner, application, reason, NULL, &local_error);
+	if (cookie == -1) {
+		tracker_dbus_request_end (request, local_error);
+
+		g_dbus_method_invocation_return_gerror (invocation, local_error);
+
+		g_error_free (local_error);
+
+		return;
+	}
+
+	tracker_dbus_request_end (request, NULL);
+	g_dbus_method_invocation_return_value (invocation,
+	                                       g_variant_new ("(i)", cookie));
+}
+
+static void
+handle_method_call_pause_for_process (TrackerMiner          *miner,
+                                      GDBusMethodInvocation *invocation,
+                                      GVariant              *parameters)
+{
+	GError *local_error = NULL;
+	gint cookie;
+	const gchar *application = NULL, *reason = NULL;
+	TrackerDBusRequest *request;
+
+	g_variant_get (parameters, "(&s&s)", &application, &reason);
+
+	tracker_gdbus_async_return_if_fail (application != NULL, invocation);
+	tracker_gdbus_async_return_if_fail (reason != NULL, invocation);
+
+	request = tracker_g_dbus_request_begin (invocation,
+	                                        "%s(application:'%s', reason:'%s')",
+	                                        __PRETTY_FUNCTION__,
+	                                        application,
+	                                        reason);
+
+	cookie = miner_pause_internal (miner,
+	                               application,
+	                               reason,
+	                               g_dbus_method_invocation_get_sender (invocation),
+	                               &local_error);
 	if (cookie == -1) {
 		tracker_dbus_request_end (request, local_error);
 
@@ -1157,20 +1358,17 @@ handle_method_call (GDBusConnection       *connection,
 
 	if (g_strcmp0 (method_name, "IgnoreNextUpdate") == 0) {
 		handle_method_call_ignore_next_update (miner, invocation, parameters);
-	} else
-	if (g_strcmp0 (method_name, "Resume") == 0) {
+	} else if (g_strcmp0 (method_name, "Resume") == 0) {
 		handle_method_call_resume (miner, invocation, parameters);
-	} else
-	if (g_strcmp0 (method_name, "Pause") == 0) {
+	} else if (g_strcmp0 (method_name, "Pause") == 0) {
 		handle_method_call_pause (miner, invocation, parameters);
-	} else
-	if (g_strcmp0 (method_name, "GetPauseDetails") == 0) {
+	} else if (g_strcmp0 (method_name, "PauseForProcess") == 0) {
+		handle_method_call_pause_for_process (miner, invocation, parameters);
+	} else if (g_strcmp0 (method_name, "GetPauseDetails") == 0) {
 		handle_method_call_get_pause_details (miner, invocation, parameters);
-	} else
-	if (g_strcmp0 (method_name, "GetProgress") == 0) {
+	} else if (g_strcmp0 (method_name, "GetProgress") == 0) {
 		handle_method_call_get_progress (miner, invocation, parameters);
-	} else
-	if (g_strcmp0 (method_name, "GetStatus") == 0) {
+	} else if (g_strcmp0 (method_name, "GetStatus") == 0) {
 		handle_method_call_get_status (miner, invocation, parameters);
 	} else {
 		g_assert_not_reached ();
